import 'package:clx_flutter_message/util/string_util.dart';
import 'package:clx_flutter_message/util/toast_util.dart';
import 'package:flutter/material.dart';

import 'clx_flutter_message_platform_interface.dart';
import 'core/model/message_config.dart';
import 'core/model/message_data.dart';
import 'core/socket/socket_io.dart';

class ClxFlutterMessage {
  Future<String?> getPlatformVersion() {
    return ClxFlutterMessagePlatform.instance.getPlatformVersion();
  }
}

MessageConfig messageConfig = MessageConfig();

abstract class BaseMessageConfig {
  //处理消息页面跳转
  void onJumpToMessagePage(String page, dynamic arguments);

  //获取未处理消息
  Future<List<MessageData>> getUnReadMessage();

  //获取未处理的公告
  Future<List<MessageData>> getUnReadNotice();

  // 标记消息为已读
  Future<bool> markReadAction(MessageData? message);

  // 删除消息
  Future<bool> removeMessageAction(MessageData? message);

  //忽略消息
  Future<bool> ignoreAction(MessageData? message);

  //标记消息已经展示
  Future<bool> setMessageShowed(MessageData? message);

  //连接websocket 获取及时消息
  Future<void> connectWebSocket() async {
    String userNo = messageConfig.userKey;
    String url = messageConfig.webSocketUrl;
    String connectId = '${DateTime.now().microsecondsSinceEpoch}-$userNo';

    var params = {
      'connectId': connectId,
      'productCode': messageConfig.productCode,
      'functionKey': messageConfig.inAppAccessKey,
      'userKey': userNo,
      'companyKey': messageConfig.companyNo
    };

    var headers = {
      'token': messageConfig.accessToken,
      'product-code': messageConfig.productCode,
    };

    Socket.getInstance().onReceivedMessage = (message) {
      // 处理消息
    };

    await Socket.getInstance().connect(url, params, headers);
  }

  // 处理消息跳转对应页面
  void gotoDealMessage(MessageData? message) {
    if (message?.canHand != true) {
      return;
    }
    if (message == null) {
      ToastUtils.showCenter("消息为空");
      return;
    }
    if (message.companyNo != messageConfig.companyNo) {
      ToastUtils.showCenter("当前公司和消息不匹配");
      return;
    }
    if (messageConfig.inAppAccessKey != message.accessKey) {
      ToastUtils.showCenter("当前角色和消息不匹配");
      return;
    }
    var data = message.textVo?.data;
    if (data == null) {
      ToastUtils.showCenter("消息数据为空");
      return;
    }
    var page = data['jumpPageAppUrl'];
    if (page == null) {
      ToastUtils.showCenter("消息跳转地址为空");
      return;
    }
    var arguments = data["jumpPageAppParam"];

    onJumpToMessagePage(page, arguments);
  }

  //刷新消息、获取未处理消息，重新连接websocket
  void refreshMessage(BuildContext context) {
    // 校验消息相关配置字段
    if (StringUtil.isEmpty(messageConfig.userKey)) {
      ToastUtils.showCenter("userNo不能为空");
      return;
    }
    if (StringUtil.isEmpty(messageConfig.companyNo)) {
      ToastUtils.showCenter("companyNo不能为空");
      return;
    }
    if (StringUtil.isEmpty(messageConfig.accessToken)) {
      ToastUtils.showCenter("登录token不能为空");

      return;
    }
    if (StringUtil.isEmpty(messageConfig.inAppAccessKey)) {
      ToastUtils.showCenter("inAppAccessKey不能为空");
      return;
    }
    if (StringUtil.isEmpty(messageConfig.productCode)) {
      ToastUtils.showCenter("productCode不能为空");
      return;
    }
    if (StringUtil.isEmpty(messageConfig.webSocketUrl)) {
      ToastUtils.showCenter("webSocketUrl不能为空");
      return;
    }

    getUnReadMessage();
    getUnReadNotice();
    connectWebSocket();
  }
}
