import 'package:flutter/material.dart';

import '../../util/string_util.dart';
import '../model/message_node.dart';
import '../widget/button_public/button_public_rect.dart';
import '../widget/button_public/button_public_type.dart';

class NoticeDialogWidget extends StatefulWidget {
  const NoticeDialogWidget({
    super.key,
    required this.showDialog,
    this.decoration,
    this.cancelText,
    this.confirmText,
    this.cancelType,
    this.confirmType,
  });

  //是否显示
  final bool showDialog;
  // 对话框背景 decoration
  final Decoration? decoration;

  // 取消按钮文字
  final String? cancelText;
  // 确认按钮文字
  final String? confirmText;

  // 取消按钮类型
  final ButtonPublicType? cancelType;
  // 确认按钮类型
  final ButtonPublicType? confirmType;

  @override
  State<NoticeDialogWidget> createState() => _NoticeDialogWidgetState();
}

class _NoticeDialogWidgetState extends State<NoticeDialogWidget> {
  MessageNode? headNode;

  @override
  Widget build(BuildContext context) {
    return _buildView();
  }

  // 构建view
  Widget _buildView() {
    var message = headNode?.data;
    var extraShowInfoList = message?.textVo?.extraShowInfoList;
    return !widget.showDialog
        ? const SizedBox()
        : Container(
            height: double.infinity,
            width: double.infinity,
            color: Colors.black.withOpacity(0.5),
            padding: const EdgeInsets.symmetric(horizontal: 20),
            child: Center(
              child: Stack(
                children: [
                  Container(
                    decoration: widget.decoration,
                    padding: const EdgeInsets.symmetric(horizontal: 24),
                    child: SingleChildScrollView(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          const SizedBox(height: 32),
                          Text(
                            message?.textVo?.title ?? '',
                            style: const TextStyle(
                                fontSize: 18, fontWeight: FontWeight.bold),
                          ),
                          const SizedBox(height: 8),
                          Text(
                            message?.textVo?.subtitle ?? '',
                            style: const TextStyle(
                                color: Color(0x99000000), fontSize: 16),
                          ),
                          if (extraShowInfoList?.isNotEmpty == true)
                            _valueContent(extraShowInfoList),
                          const SizedBox(height: 24),
                          Row(
                            children: [
                              Expanded(
                                child: ButtonPublicRect(
                                  type: widget.cancelType ??
                                      ButtonPublicType.outline,
                                  text: widget.cancelText ?? '忽略',
                                  onPressed: ignore,
                                  borderRadius: 8,
                                ),
                              ),
                              const SizedBox(width: 10),
                              Expanded(
                                child: ButtonPublicRect(
                                  type: widget.confirmType ??
                                      ButtonPublicType.solid,
                                  text: widget.confirmText ?? '去处理',
                                  onPressed: () {
                                    MessageDeal.gotoDealMessage(message);
                                    ignore();
                                  },
                                  borderRadius: 8,
                                ),
                              )
                            ],
                          ),
                          const SizedBox(height: 24),
                        ],
                      ),
                    ),
                  ),
                  Positioned(
                    top: 0,
                    right: 0,
                    child: IconButton(
                      onPressed: dismiss,
                      icon: const Icon(
                        Icons.close,
                        size: 16,
                        color: Colors.grey,
                      ),
                    ),
                  )
                ],
              ),
            ),
          );
  }

  Container _valueContent(List<dynamic>? extraShowInfoList) {
    return Container(
      padding: const EdgeInsets.only(left: 10, right: 10, bottom: 10),
      margin: const EdgeInsets.only(top: 8),
      decoration: BoxDecoration(
          color: const Color(0xFFF3F5F8),
          borderRadius: BorderRadius.circular(8)),
      child: Column(
        children: extraShowInfoList!.map<Widget>((e) {
          return Container(
            margin: const EdgeInsets.only(top: 10),
            child: Row(
              children: [
                SizedBox(
                  width: 75,
                  child: Text(
                    (e['name'] ?? '') + "：",
                    style: const TextStyle(
                      color: Color(0xFF4E5969),
                      fontSize: 15,
                    ),
                  ),
                ),
                const SizedBox(width: 10),
                Expanded(
                  child: Row(
                    children: [
                      Flexible(
                        child: Text(
                          StringUtil.formatValue(e['value']),
                          style: const TextStyle(
                            color: Color(0xFF333C4C),
                            fontSize: 15,
                          ),
                          overflow: TextOverflow.ellipsis,
                        ),
                      ),
                      const SizedBox(
                        width: 10,
                      ),
                      Text(
                        StringUtil.formatValue(e['unit'], defaultValue: ''),
                        style: const TextStyle(
                          color: Color(0xFF4E5969),
                          fontSize: 14,
                        ),
                      ),
                    ],
                  ),
                ),
              ],
            ),
          );
        }).toList(),
      ),
    );
  }

  void dismiss() {
    MessageDeal.setMessageShowed(headNode?.data);
    next();
  }

  void next() {
    headNode = headNode?.next;

    /// 隐藏当前的
    showDialog = false;
    update();

    /// 显示下一个
    if (headNode != null) {
      Future.delayed(const Duration(milliseconds: 300)).then((value) {
        showDialog = true;
        update();
      });
    }
  }

  void ignore() {
    /// 标记已展示
    MessageDeal.setMessageShowed(headNode?.data);
    MessageDeal.markReadAction(headNode?.data);
    next();
  }
}
