import 'package:clx_flutter_message/clx_flutter_message.dart';

import '../../model/message_data.dart';
import '../../model/message_node.dart';

mixin NotificationLayoutLogic {
  MessageNode? headNode;

  BaseMessageConfig? messageDeal;

  void setCashMessage(List<MessageData> messages) {
    headNode = null;

    /// 上一个节点
    MessageNode? preNode;
    for (var data in messages) {
      var theNode = MessageNode(data);
      if (preNode == null) {
        preNode = theNode;
        headNode = preNode;
      } else {
        preNode.next = theNode;
        theNode.prev = preNode;
        preNode = theNode;
      }
    }
  }

  void next() {
    messageConfig.baseMessageConfig?.setMessageShowed(headNode?.data);
    if (headNode?.next == null) {
      headNode = null;
      NotificationManager.instance.hideNotification();
      return;
    }
    headNode = headNode?.next;
    headNode?.prev?.next = null;
    headNode?.prev = null;
    update();
  }

  void ignore() {
    MessageDeal.markReadAction(headNode?.data);
    MessageDeal.setMessageShowed(headNode?.data);
    next();
  }

  void dismiss() {
    MessageDeal.setMessageShowed(headNode?.data);
    next();
  }

  void confirm(MessageData data) {
    messageDeal?.gotoDealMessage(data);
    messageDeal?.markReadAction(headNode?.data);
    messageDeal?.setMessageShowed(headNode?.data);
    next();
  }

  /// 插入消息
  void insertNotification(MessageData data) {
    if (headNode == null) {
      headNode = MessageNode(data);
    } else {
      headNode?.insertData(data);
    }
  }

  void clear() {
    headNode = null;
    NotificationManager.instance.hideNotification();
    update();
  }
}
