import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../../model/message_data.dart';
import '../../model/message_node.dart';
import 'notification_layout_logic.dart';
import 'widget/list_item.dart';

class NotificationLayoutWidget extends StatefulWidget {
  const NotificationLayoutWidget({super.key});

  @override
  State<NotificationLayoutWidget> createState() =>
      _NotificationLayoutWidgetState();
}

class _NotificationLayoutWidgetState extends State<NotificationLayoutWidget> {
  MessageNode? headNode;
  @override
  Widget build(BuildContext context) {
    return _buildView();
  }

  // 构建View
  Widget _buildView() {
    if (headNode == null) {
      return const SizedBox();
    }
    return Container(
      margin: const EdgeInsets.only(top: 35),
      child: buildStackedList(context),
    );
  }

  Widget buildStackedList(BuildContext context) {
    return SizedBox(
      height: 140,
      child: Stack(
        fit: StackFit.expand,
        children: [
          if (headNode?.next != null)
            Positioned(
              bottom: 15,
              left: 25,
              right: 25,
              child: _message(headNode?.next?.data),
            ),
          Positioned(
            bottom: 0,
            left: 10,
            right: 10,
            child: _buildNotification(headNode?.data),
          )
        ],
      ),
    );
  }

  MessageWidget _message(data) {
    return MessageWidget(
      data: data,
      width: double.infinity,
      onCancel: () {
        ignore();
      },
      onHand: () {
        confirm(data);
      },
    );
  }

  Widget _buildNotification(data) {
    if (data == null) {
      return const SizedBox();
    }
    return Dismissible(
      key: Key(data.messageNo?.toString() ?? ''),
      direction: DismissDirection.endToStart,
      onDismissed: (direction) {
        dismiss();
      },
      child: Stack(
        children: [
          Positioned.fill(
            child: ClipRRect(
              borderRadius: BorderRadius.circular(8),
              child: BackdropFilter(
                filter: ImageFilter.blur(sigmaX: 1.5, sigmaY: 1.5),
                child: Container(
                  decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(8),
                      color: Colors.black26),
                  width: null,
                  height: null,
                ),
              ),
            ),
          ),
          _message(data),
        ],
      ),
    );
  }

  void ignore() {
    MessageDeal.markReadAction(headNode?.data);
    MessageDeal.setMessageShowed(headNode?.data);
    next();
  }

  void next() {
    MessageDeal.setMessageShowed(headNode?.data);
    if (headNode?.next == null) {
      headNode = null;
      hideNotification();
      return;
    }
    headNode = headNode?.next;
    headNode?.prev?.next = null;
    headNode?.prev = null;
    setState(() {});
  }

  void clear() {
    headNode = null;
    hideNotification();
    setState(() {});
  }

  void dismiss() {
    MessageDeal.setMessageShowed(headNode?.data);
    next();
  }

  void setCashMessage(List<MessageData> messages) {
    headNode = null;

    /// 上一个节点
    MessageNode? preNode;
    for (var data in messages) {
      var theNode = MessageNode(data);
      if (preNode == null) {
        preNode = theNode;
        headNode = preNode;
      } else {
        preNode.next = theNode;
        theNode.prev = preNode;
        preNode = theNode;
      }
    }
  }

  /// 插入消息
  void insertNotification(MessageData data) {
    if (headNode == null) {
      headNode = MessageNode(data);
    } else {
      headNode?.insertData(data);
    }
  }



  void confirm(MessageData data) {
    MessageDeal.gotoDealMessage(data);
    MessageDeal.markReadAction(headNode?.data);
    MessageDeal.setMessageShowed(headNode?.data);
    next();
  }

    void setNotification(List<dynamic> data) {
    setCashMessage(
      data.map<MessageData>((e) => MessageData.fromJson(e)).toList(),
    );
  }

}



class NotePage extends GetView<NotificationLayoutLogic> {
  const NotePage({super.key});

  // 主视图
  Widget _buildView() {
    return const Center(
      child: Text("NotePage"),
    );
  }

  @override
  Widget build(BuildContext context) {
    return GetBuilder<NoteController>(
      init: NoteController(),
      id: "note",
      builder: (_) {
        return Scaffold(
          appBar: AppBar(title: const Text("note")),
          body: SafeArea(
            child: _buildView(),
          ),
        );
      },
    );
  }
}
