import 'package:flutter/cupertino.dart';

import '../model/message_data.dart';

/// 全局消息展示管理
class NotificationManager {
  static NotificationManager? _instance;

  static NotificationManager get instance {
    _instance ??= NotificationManager._internal();
    return _instance!;
  }

 

  OverlayEntry? _overlayEntry;

  

  /// 显示悬浮布局
  void showNotification(BuildContext? context) {
    if (context == null) return;
    if (_overlayEntry != null) return; // 防止重复显示
    if (_notificationLayoutLogic.headNode == null) return; // 防止没有消息时显示
    _overlayEntry = OverlayEntry(
      builder: (context) => Positioned(
        top: 0,
        left: 0,
        right: 0,
        child: NotificationLayoutPage(
          logic: _notificationLayoutLogic,
        ),
      ),
    );

    Overlay.of(context).insert(_overlayEntry!);
  }

  /// 隐藏悬浮布局
  void hideNotification() {
    _overlayEntry?.remove();
    _overlayEntry = null;
  }

  /// 缓存之前的消息列表
  void setNotification(List<dynamic> data) {
    _notificationLayoutLogic.setCashMessage(
      data.map<MessageData>((e) => MessageData.fromJson(e)).toList(),
    );
  }

  void insertNotification(MessageData data) {
    _notificationLayoutLogic.insertNotification(data);
  }

  void clear() {
    _notificationLayoutLogic.clear();
  }
}
